Class: TurnKit::Turn

Inherits:
Object
  • Object
show all
Defined in:
lib/turnkit/turn.rb

Constant Summary collapse

STATUSES =
Record::TURN_STATUSES

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(agent:, conversation:, record:, store:, budget: nil, depth: 0, on_event: nil) ⇒ Turn

Returns a new instance of Turn.



12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
# File 'lib/turnkit/turn.rb', line 12

def initialize(agent:, conversation:, record:, store:, budget: nil, depth: 0, on_event: nil)
  @agent = agent
  @conversation = conversation
  @store = store
  @record = record.transform_keys(&:to_s)
  @id = @record.fetch("id")
  @conversation_id = @record.fetch("conversation_id")
  @agent_name = @record["agent_name"]
  @parent_turn_id = @record["parent_turn_id"]
  @parent_tool_execution_id = @record["parent_tool_execution_id"]
  @root_turn_id = @record["root_turn_id"] || id
  @context_message_sequence = @record["context_message_sequence"].to_i
  @model = @record["model"] || agent.effective_model
  @thinking = thinking_from_options
  @compact = compact_from_options
  @output_schema = output_schema_from_options
  @prompt_mode = prompt_mode_from_options
  @started_at = @record["started_at"]
  @budget = budget || agent.build_budget
  @depth = depth
  @on_event = on_event
end

Instance Attribute Details

#agentObject (readonly)

Returns the value of attribute agent.



7
8
9
# File 'lib/turnkit/turn.rb', line 7

def agent
  @agent
end

#agent_nameObject (readonly)

Returns the value of attribute agent_name.



8
9
10
# File 'lib/turnkit/turn.rb', line 8

def agent_name
  @agent_name
end

#budgetObject (readonly)

Returns the value of attribute budget.



7
8
9
# File 'lib/turnkit/turn.rb', line 7

def budget
  @budget
end

#compactObject (readonly)

Returns the value of attribute compact.



9
10
11
# File 'lib/turnkit/turn.rb', line 9

def compact
  @compact
end

#context_message_sequenceObject (readonly)

Returns the value of attribute context_message_sequence.



9
10
11
# File 'lib/turnkit/turn.rb', line 9

def context_message_sequence
  @context_message_sequence
end

#conversationObject (readonly)

Returns the value of attribute conversation.



7
8
9
# File 'lib/turnkit/turn.rb', line 7

def conversation
  @conversation
end

#conversation_idObject (readonly)

Returns the value of attribute conversation_id.



8
9
10
# File 'lib/turnkit/turn.rb', line 8

def conversation_id
  @conversation_id
end

#depthObject (readonly)

Returns the value of attribute depth.



7
8
9
# File 'lib/turnkit/turn.rb', line 7

def depth
  @depth
end

#idObject (readonly)

Returns the value of attribute id.



8
9
10
# File 'lib/turnkit/turn.rb', line 8

def id
  @id
end

#modelObject (readonly)

Returns the value of attribute model.



9
10
11
# File 'lib/turnkit/turn.rb', line 9

def model
  @model
end

#output_schemaObject (readonly)

Returns the value of attribute output_schema.



9
10
11
# File 'lib/turnkit/turn.rb', line 9

def output_schema
  @output_schema
end

#parent_tool_execution_idObject (readonly)

Returns the value of attribute parent_tool_execution_id.



8
9
10
# File 'lib/turnkit/turn.rb', line 8

def parent_tool_execution_id
  @parent_tool_execution_id
end

#parent_turn_idObject (readonly)

Returns the value of attribute parent_turn_id.



8
9
10
# File 'lib/turnkit/turn.rb', line 8

def parent_turn_id
  @parent_turn_id
end

#prompt_modeObject (readonly)

Returns the value of attribute prompt_mode.



9
10
11
# File 'lib/turnkit/turn.rb', line 9

def prompt_mode
  @prompt_mode
end

#root_turn_idObject (readonly)

Returns the value of attribute root_turn_id.



9
10
11
# File 'lib/turnkit/turn.rb', line 9

def root_turn_id
  @root_turn_id
end

#started_atObject (readonly)

Returns the value of attribute started_at.



10
11
12
# File 'lib/turnkit/turn.rb', line 10

def started_at
  @started_at
end

#storeObject (readonly)

Returns the value of attribute store.



7
8
9
# File 'lib/turnkit/turn.rb', line 7

def store
  @store
end

#thinkingObject (readonly)

Returns the value of attribute thinking.



9
10
11
# File 'lib/turnkit/turn.rb', line 9

def thinking
  @thinking
end

Class Method Details

.iterations_for(record) ⇒ Object

Reads iterations from options, falling back to the legacy top-level key for turns persisted before the state split.



122
123
124
125
# File 'lib/turnkit/turn.rb', line 122

def self.iterations_for(record)
  options = record["options"] || {}
  (options.dig("state", "iterations") || options["iterations"]).to_i
end

Instance Method Details

#costObject



131
132
133
# File 'lib/turnkit/turn.rb', line 131

def cost
  Cost.from_record(@record)
end

#emit(type, payload = {}) ⇒ Object



152
153
154
# File 'lib/turnkit/turn.rb', line 152

def emit(type, payload = {})
  emit_event(Event.new(type: type, turn_id: id, conversation_id: conversation.id, payload: payload))
end

#internal_model_call(model:, messages:, instructions:, tools: [], thinking: nil, output_schema: nil, metadata: {}, purpose:, client: nil) ⇒ Object



156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
# File 'lib/turnkit/turn.rb', line 156

def internal_model_call(model:, messages:, instructions:, tools: [], thinking: nil, output_schema: nil, metadata: {}, purpose:, client: nil)
  request = ModelRequest.new(
    model: model,
    messages: messages,
    tools: tools,
    instructions: instructions,
    thinking: thinking,
    output_schema: output_schema,
    metadata: { purpose: purpose.to_s, turn_id: id, conversation_id: conversation.id }.merge( || {})
  )
  model_client = client || agent.effective_client
  model_client.validate!(model: request.model)

  emit_model_requested("#{purpose}.model.requested", request)
  result = call_client(request, client: model_client)
  result_cost = Cost.from_usage(result.usage, model: result.model || request.model)
  add_usage!(result.usage, cost: result_cost)
  emit_model_completed("#{purpose}.model.completed", result, result_cost, model: request.model)
  budget.add_cost!(result_cost.total)
  result
end

#output_dataObject



111
112
113
# File 'lib/turnkit/turn.rb', line 111

def output_data
  @record["output_data"]
end

#output_textObject



107
108
109
# File 'lib/turnkit/turn.rb', line 107

def output_text
  @record["output_text"].to_s
end

#paint(prompt, model:, provider: nil, size: nil, assume_model_exists: nil, input_images: nil, mask: nil, params: {}, metadata: {}, client: nil) ⇒ Object



178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
# File 'lib/turnkit/turn.rb', line 178

def paint(prompt, model:, provider: nil, size: nil, assume_model_exists: nil, input_images: nil, mask: nil, params: {}, metadata: {}, client: nil)
  claimed = claim_standalone!("paint")
  if claimed
    @record = claimed
    @started_at = @record["started_at"]
    emit("turn.started", status: status, model: model)
    @budget = Budget.resume(store: store, root_turn_id: root_turn_id, limits: agent.budget_limits)
  end
  image_client = client || agent.effective_client
  request = {
    prompt: prompt,
    model: model,
    provider: provider,
    size: size,
    assume_model_exists: assume_model_exists,
    input_images: input_images,
    mask: mask,
    params: params || {},
    metadata: { turn_id: id, conversation_id: conversation.id }.merge( || {})
  }

  image_client.validate!(model: model)
  emit("image.requested", request.except(:input_images, :mask))
  result = call_image_client(image_client, request)
  result_cost = apply_result_cost(result, model: model)
  image = result.images.first
  raise Error, "image client returned no image" unless image
  raise Error, "image client returned image without url or data" if image.url.to_s.empty? && image.data.to_s.empty?

  persist_image_message(image)
  emit("image.completed", image: image.to_h, model: image.model || model, provider: image.provider || provider&.to_s, mime_type: image.mime_type, usage: result.usage.to_h, cost: result_cost.to_h, metadata:  || {})
  complete_with_output(image.url.to_s, output_data: { "type" => "image", "images" => [ image.to_h ] }, audit: check_policy(image.url.to_s, output_data: { "type" => "image", "images" => [ image.to_h ] })) if claimed
  image
rescue StandardError => error
  fail_standalone!(error) if claimed
  raise
end

#policy_auditObject



115
116
117
118
# File 'lib/turnkit/turn.rb', line 115

def policy_audit
  options = @record["options"] || {}
  options.dig("state", "policy_audit") || options["policy_audit"]
end

#previewObject



95
96
97
# File 'lib/turnkit/turn.rb', line 95

def preview
  model_request
end

#reloadObject



139
140
141
142
143
144
145
146
# File 'lib/turnkit/turn.rb', line 139

def reload
  @record = store.load_turn(id)
  @thinking = thinking_from_options
  @compact = compact_from_options
  @output_schema = output_schema_from_options
  @prompt_mode = prompt_mode_from_options
  self
end

#run!(&block) ⇒ Object



35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
# File 'lib/turnkit/turn.rb', line 35

def run!(&block)
  @on_event = block if block
  return self unless status == "pending"

  claimed = store.claim_turn(id, from: "pending", to: "running", started_at: Clock.now, heartbeat_at: Clock.now)
  return self unless claimed

  @record = claimed
  @started_at = @record["started_at"]
  emit("turn.started", status: status, model: model)
  agent.effective_client.validate!(model: model)
  @budget = Budget.resume(store: store, root_turn_id: root_turn_id, limits: agent.budget_limits)
  revisions_used = 0
  loop do
    budget.check!(depth: depth)
    count_iteration!
    TurnKit::Compaction.maybe_compact!(self)

    request = model_request
    emit_model_requested("model.requested", request)
    result = call_client(request)
    result_cost = Cost.from_usage(result.usage, model: result.model || model)

    add_usage!(result.usage, cost: result_cost)
    emit_model_completed("model.completed", result, result_cost, model: model)
    budget.add_cost!(result_cost.total)
    persist_assistant_message(result)

    if result.tool_calls?
      runner = ToolRunner.new(self)
      terminal = runner.dispatch(result.tool_calls)
      if terminal
        candidate = append_terminal_completion(runner, terminal)
      else
        next
      end
    else
      candidate = result.text
    end

    audit = check_policy(candidate, output_data: result.output_data)
    if should_revise?(audit, revisions_used)
      revisions_used += 1
      append_revision_message(audit, attempt: revisions_used, terminal_tool_name: terminal&.tool_name)
      emit("output_policy.revision", violation_count: audit.violations.length, attempt: revisions_used)
      next
    end

    complete_with_output(candidate, output_data: result.output_data, audit: audit)
    break
  end
  reload
  self
rescue StandardError => error
  update!(status: "failed", error: { "class" => error.class.name, "message" => error.message }, completed_at: Clock.now)
  emit("turn.failed", error: { "class" => error.class.name, "message" => error.message })
  reload
  self
end

#stale!Object



148
149
150
# File 'lib/turnkit/turn.rb', line 148

def stale!
  update!(status: "stale", completed_at: Clock.now)
end

#statusObject



99
100
101
# File 'lib/turnkit/turn.rb', line 99

def status
  @record.fetch("status")
end

#tool_executionsObject



135
136
137
# File 'lib/turnkit/turn.rb', line 135

def tool_executions
  store.list_tool_executions(turn_id: id).map { |attrs| ToolExecution.new(attrs) }
end

#usageObject



127
128
129
# File 'lib/turnkit/turn.rb', line 127

def usage
  Usage.from_h(@record["usage"] || {})
end

#view_media(media, objective:, model:, provider: nil, output_schema: nil, params: {}, metadata: {}, client: nil) ⇒ Object



216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
# File 'lib/turnkit/turn.rb', line 216

def view_media(media, objective:, model:, provider: nil, output_schema: nil, params: {}, metadata: {}, client: nil)
  claimed = claim_standalone!("view media")
  if claimed
    @record = claimed
    @started_at = @record["started_at"]
    emit("turn.started", status: status, model: model)
    @budget = Budget.resume(store: store, root_turn_id: root_turn_id, limits: agent.budget_limits)
  end
  media_input = MediaInput.wrap(media)
  media_client = client || agent.effective_client
  request = {
    media: media_input,
    objective: objective,
    model: model,
    provider: provider,
    output_schema: output_schema,
    params: params || {},
    metadata: { turn_id: id, conversation_id: conversation.id }.merge( || {})
  }

  media_client.validate!(model: model)
  emit("media.requested", request.except(:media).merge(media: media_input.to_h))
  result = call_media_client(media_client, request)
  result_cost = apply_result_cost(result, model: model)
  analysis = result.media_analyses.first
  raise Error, "media client returned no media analysis" unless analysis

  persist_media_analysis_message(analysis)
  output_data = { "type" => "media_analysis", "media_analyses" => [ analysis.to_h ] }
  emit("media.completed", analysis: analysis.to_h, model: analysis.model || model, provider: analysis.provider || provider&.to_s, media: media_input.to_h, usage: result.usage.to_h, cost: result_cost.to_h, metadata:  || {})
  complete_with_output(analysis.text, output_data: output_data, audit: check_policy(analysis.text, output_data: output_data)) if claimed
  analysis
rescue StandardError => error
  emit("media.failed", error: { "class" => error.class.name, "message" => error.message }, metadata:  || {}) if status == "running" || claimed
  fail_standalone!(error) if claimed
  raise
end